!Programa que calcula la deconvolucion tensorial en 3 D de datos de tensor gravimtrico completo
!FTG full tensor gradiometry por sus siglas en ingls 
!programa elaborado siguiendo los principios de Mikhailov, ET Al 2007, Geophysics
!Usare el modulo Euler, desarrollado por el M. Mauricio Nava Flores para el llamado de
!datos desde el archivo y guardado, pero se usaran funciones de nuevas librerias y librerias de sistema
!que no estaban incluidas previamente.
!
!
!
!Las unidades bajo las cuales se dise el programa son:
!
!- Longitud:              [m]
!- Anomala gravimtrica: [mGal]
!- Anomala magntica:    [nT]
!


program nucleodectens

use Eulermio
!use omp_lib

implicit none

double precision, dimension(3) :: hj



call system('clear')

write(*,10)

10 FORMAT('----------------------------------------------------------------------',/,&
&'    Programa que calcula la deconvolucin tensorial     ',/,&
&'                      para datos en malla regular                     ',/,&
&'----------------------------------------------------------------------',/,&
&'  - Algoritmo basado en la metodologia de Mikhailov, ET AL (2007).        ',/,&
&'  - Se usa un cono de afectacion para la seleccion de datos.        ',/,&
&'  - Las mallas de datos deben estar almacenadas en forma matricial. ',/,&
&'  - Se genera un archivo .dat como salida, para ser graficado con puntos  ',/,&
&'  - Ocupa el modulo Eulermio, que es una nueva versin del creado por el MC. Mauricio Nava',/,&
&'----------------------------------------------------------------------')



!nucleo del programa, donde se reaalizaran las operaciones necesarias para la obtencin de los
!eigenvectores y eigenvalores, ademas de la profundidad estimada
!se definen los nombres de la variables, se necesita armar nuestro tensor de tres por tres, vamos a 
!intentar manejarlo como matriz para obtener los eigenvectores y manejemos cada uno de sus componentes 
!por su nombre en la obtencion de los eigen valores


! Se mandan llamar los archivos con la informacion, y se cargaran en un vector cada componente
!
!--------------------------------------------------------------------------------------------------!
!                     PARMETROS DE LA DECONVOLUCIN TENSORIAL - ARCHIVO DE TEXTO
!--------------------------------------------------------------------------------------------------!
Archtxt='DE3D_Param.txt'
call LectorParam(Archtxt,Xini,Yini,dx,dy,N,Tol,V1)
!-----------------


!1.- Carga de anomala

Malla2D='Gz.dat'
call Lector_1D_2D(Malla2D,Gz,Ny,Nx)

!2.- Carga de gradientes
Malla2D='Gxx.dat'
call Lector_1D_2D(Malla2D,Gxx,Ny,Nx)
Malla2D='Gxy.dat'
call Lector_1D_2D(Malla2D,Gxy,Ny,Nx)
Malla2D='Gxz.dat'
call Lector_1D_2D(Malla2D,Gxz,Ny,Nx)
Malla2D='Gyy.dat'
call Lector_1D_2D(Malla2D,Gyy,Ny,Nx)
Malla2D='Gyz.dat'
call Lector_1D_2D(Malla2D,Gyz,Ny,Nx)
Malla2D='Gzz.dat'
call Lector_1D_2D(Malla2D,Gzz,Ny,Nx)
!-----------
!Se llama una mejora en el uso del procesaor
call cpu_time(tini)


!--------------------------------------------------------------------------------------------------!
!                             EJES COORDENADOS: (j -> X)  (i -> Y)
!--------------------------------------------------------------------------------------------------!
allocate(XCoord(Nx),YCoord(Ny))
do j=1,Nx
XCoord(j)=Xini+(j-1)*dx
end do
do i=1,Ny
YCoord(i)=Yini+(i-1)*dy
end do
!-------
!sabemos que los valores del triangulo inferior y superior, dadas las propiedades de
!los tensores son iguales,por lo que solo se repetira el valor


!---------   Arreglos que se van a ocupar para la dec tens, seal analitica y radio invarante

allocate(A(3,3),v(3,3),sax(nx,ny),say(nx,ny),saz(nx,ny),rinv(nx,ny))
allocate(d(3),exis(Nx*Ny),ygri(Nx*Ny),zeta(Nx*Ny))
num=0.0!!checar


!------------------------Se inicia el calculo de la solucion punto  a punto

do j=1,Nx
  do i=1,Ny

    ! A   seria hacerla ceros, hay qe ver si vale la pena
     A(1,1)=Gxx(j,i)
     A(1,2)=Gxy(j,i)
     A(1,3)=Gxz(j,i)
     A(2,1)=Gxy(j,i)
     A(2,2)=Gyy(j,i)
     A(2,3)=Gyz(j,i)
     A(3,1)=Gxz(j,i)
     A(3,2)=Gyz(j,i)
     A(3,3)=Gzz(j,i)



     call jacobi(A,d,v,nrot)     !Se calcula el radio invariante


!                                declaradas igual en el modulo
    
  i1=d(1)+d(2)+d(3)
  i2=d(1)*d(2)+d(2)*d(3)+d(1)*d(3)
  i3=d(1)*d(2)*d(3)
  inva=-((i3/2)**2)/((i2/3)**3)
! se genera la salida para la seal analtica y el rdio invariante

rinv(j,i)=inva  !ina matriz para graficar el radio invariante

sax(j,i)=dsqrt((Gxx(j,i)**2+Gxy(j,i)**2+Gxz(j,i)**2))  ! aqui se calcula la seal analtica
say(j,i)=dsqrt((Gyy(j,i)**2+Gxy(j,i)**2+Gyz(j,i)**2))
saz(j,i)=dsqrt((Gzz(j,i)**2+Gxz(j,i)**2+Gyz(j,i)**2))

 call eigsrt(d,v) ! se acomodan los vectores de mayor a menor para seleccionar los respectivos al eigen valor usado


   ! Ahora, usando el valor de d que sea el maximo absoluto, calculamos nuestra profundidad


      zet=(1+inva)*Gz(j,i)/maxval(abs(d))



!  usando parametrizacin de la recta con un vector director y un punto ,
!   calculamos los otros dos componentes
      tem=zet/v(1,3)
      xet=XCoord(j)+(tem*v(1,1))
      yet=YCoord(i)+(tem*v(1,2))


!! analizamos a ver si pasa el criterio de aceptacin
    di2=(xet-XCoord(j))**2+(yet-YCoord(i))**2
    ze2=zet**2
    if ( di2 < ze2)  then  !  si es menor la distancia horizontal
!         que la profundidad son aceptados, 
!                   Fitzgerald, 2004
        !! se lleva un contador
        if (10000>zet .and. zet>1) then
              num=num+1
       write(*,*) num, zet, inva
       exis(num)=xet
       ygri(num)=yet !se asignan los valores a nuestros vectores solucin
       zeta(num)=zet
     !! else
       !! regresar todo a cero, como inicializar una matriz a cero
end if
end if
end do
end do

! creamos los archivos de salida

open(unit=200,FILE='salidaxyzdectens.dat',status='replace',action='write' )
do l=1,num
     write(200,*) exis(l), ygri(l), zeta(l)
end do
close(200)

write(*,15)
15 format(/,'Seal analtica X')
call Esc_Result(sax,Nx,Ny,dx,dy,Xini,Yini)
write(*,16)
16 format(/,'Seal analtica Y')
call Esc_Result(say,Nx,Ny,dx,dy,Xini,Yini)
write(*,17)
17 format(/,'Seal analtica Z')
call Esc_Result(saz,Nx,Ny,dx,dy,Xini,Yini)
write(*,18)
18 format(/,'Radio Invariante')
call Esc_Result(rinv,Nx,Ny,dx,dy,Xini,Yini)

! igual aqui conviene poner los deallocate

!  deallocate (todo)
end program nucleodectens






