!Módulo Euler
!Módulo que contiene todas las declaraciones de variables y subrutinas necesarias para el cálculo 
!de la solución de la ecuación de Euler y todas sus variantes.
!
!Las subrutinas y funciones presentes en el módulo Euler son:
!
!   1.- Lector_1D_2D: Lector de arreglos 1D y 2D.
!   2.- Esc_Result.
!   3.- EscSolgnuplot.
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
module Eulermio


implicit none
integer:: nA,m,ninterv,nind,p,icorrel,hilos,V1
integer*8:: Nx,Ny,i,j,k,l,acept,nVent,num,nrot
double precision :: tini,tfin,Xini,Yini,dx,dy,N,Tol,Ka,xMv,yMv,Ll,sz,coc,xVmin,xVmax,yVmin,yVmax, &
                   pcAcept,di2,ze2,tcomp,nini,nfin,deltan,i1,i2,i3,inva,zet,xet,yet,tem
double precision, allocatable, dimension(:) :: XCoord,YCoord,X,B,dec,Nn,BB,HH,d,zeta,exis,ygri
double precision, allocatable, dimension(:,:) :: Gz,Gxz,Gyz,Gzz,Gzy,Gzx,A,Ainv,Atr,Ata, &
                                                 AinvAtr,SolT,SolAcept,Correl,Gxx,Gxy,Gyy,&
                                                 Gx,Gy,Gyx,v,sax,say,saz,rinv
logical:: pasa
 character (len=55):: Malla2D,Archtxt,ResXYZ,indice,tolerancia,ventana

contains

!---------------------------------------------------------------------------------------------------
!Subrutina Lector_1D_2D:
!Útil para leer archivos ASCII de datos correspondientes a matrices o arreglos 1D y 2D.
!
!La subrutina Lector_1D_2D, carga el archivo de datos y lo almacena en un arreglo 2D de números de 
!doble precisión, listo para ser utilizado por el programa de propagación de ondas 2D.
!
!No es necesario introducir las dimensiones de las matrices por leer, la subrutina las determina
!automáticamente.
!
!Parámetros de entrada:
!- Nombre del archivo por leer (cadena de caracteres): Archivo2D
!- Nombre de la variable (arreglo) en que se almacenarán los de datos cargados del archivo leído: 
!  Arr2D
!
!Parámetros de salida:
!- Dimensiones del arreglo: Nren, Ncol
!- Arreglo con los datos cargados: Datos2D
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se debe tener 
!   cuidado con las variables y parámetros de entrada y salida de la misma>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine Lector_1D_2D(Archivo2D,Arr2D,Ny,Nx)
implicit none
 character (len=35):: Archivo2D
integer*8, intent(out):: Ny,Nx
integer*8:: i,j,Nren,Ncol
double precision:: buffer1, buffer2
double precision, allocatable, dimension(:,:):: Arr2D,Arr2D1,Arr2D2
Nren=100000000
Ncol=Nren

open(unit=100,file=Archivo2D,status='old',action='read')
do i=1,Nren
   read(100,*,end=1000) buffer1
end do
1000 Nren=i-1
 close(100)

open(unit=100,file=Archivo2D,status='old',action='read')
read(100,*,end=2000)(buffer2,j=1,Ncol*Nren)
2000 Ncol=(j-1)/Nren
 close(100)

open(unit=100,file=Archivo2D,status='old',action='read')
allocate(Arr2D1(Nren,Ncol))
do i=1,Nren
   read(100,*,end=3000)(Arr2D1(i,j),j=1,Ncol)
end do
3000 continue
 close(100)

write(*,10) trim(Archivo2D),Ncol,Nren
10 format( '                                 Archivo: ',a,'             ',/,&
          &'Dimensiones de la malla de datos cargada: ',i5,' Muestras en X',/,&
          &'                                          ',i5,' Muestras en Y')

if (Ncol==1) then
   write(*,11)
   read(*,*) Ncol
   write(*,12)
   read(*,*) Nren
   allocate(Arr2D(Nren,Ncol),Arr2D2(Nren,Ncol))
   Arr2D2=reshape(Arr2D1,(/Nren,Ncol/))
   Arr2D=Arr2D2
   deallocate(Arr2D1,Arr2D2)
else if (Ncol/=1) then
   allocate(Arr2D(Nren,Ncol))
   Arr2D=Arr2D1
   deallocate(Arr2D1)
end if

11 format(/,'-----------------------------------------------------------',/,&
          & '            DIMENSIONES DE LAS MALLAS DE DATOS             ',/,&
          & '-----------------------------------------------------------',/,&
          & 'Datos en X  (Columnas):                                    ')

12 format(/ 'Datos en Z (Renglones):                                    ')

Ny=Nren
Nx=Ncol
return
end subroutine Lector_1D_2D
!---------------------------------------------------------------------------------------------------

!---------------------------------------------------------------------------------------------------
!Subrutina LectorParam:
!Útil para leer archivos ASCII con información de parámetros para la Deconvolución de Euler 3D
!
!La subrutina LectorParam, carga el archivo de texto y carga en las variables determinadas, los
!parámetros para la deconvolución de Euler
!
!Parámetros de entrada:
!- Nombre del archivo por leer (cadena de caracteres): Archtxt
!
!Parámetros de salida:
!- Coordenadas iniciales del vertice inferior izquierdo: (Xini,Yini)
!- Incrementos de muestreo en X y Y: dx, dy
!- Indice Estructural: N
!- Tolerancia: Tol
!- Ancho de ventana móvil: V - puntos de malla
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se debe tener 
!   cuidado con las variables y parámetros de entrada y salida de la misma>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine LectorParam(Archtxt,Xini,Yini,dx,dy,N,Tol,V1)
implicit none
character (len=80):: Enc
character (len=35):: Archtxt
character (len=9):: c1
integer:: i
integer, intent(out):: V1
double precision, intent(out):: Xini,Yini,dx,dy,N,Tol

open(unit=100,file=Archtxt,status='old',action='read')
do i=1,7
read(100,'(a80)') Enc
end do
read(100,10) c1,Xini,c1,Yini,c1,dx,c1,dy,c1,N,c1,Tol,c1,V1
close(100)

write(*,11) Xini,Yini,dx,dy,N,Tol,V

10 format(6(a6,f10.1,/),a6,i2)
11 FORMAT(/,'----------------------------------------------------------------------',/,&
&'          PARAMETROS DEL METODO DE DECONVOLUCION DE EULER 3D          ',/,&
&'----------------------------------------------------------------------',/,&
&'                 Coordenada inicial en X: ',f8.3,'                    ',/,&
&'                 Coordenada inicial en Y: ',f8.3,'                    ',/,&
&'              Intervalo de muestreo en X: ',f8.3,'                    ',/,&
&'              Intervalo de muestreo en Y: ',f8.3,'                    ',/,&
&'                      Indice Estructural: ',f8.3,'                    ',/,&
&'                              Tolerancia: ',f8.3,'                    ',/,&
&'                        Ancho de ventana: ',i8,'                      ',/,&
&'----------------------------------------------------------------------',/)

return
end subroutine LectorParam
!---------------------------------------------------------------------------------------------------

!---------------------------------------------------------------------------------------------------
!Subrutina invChol:
!Útil para calcular la matriz inversa por descomposición de Cholesky
!
!Algoritmo basado en la librería creada por Vadim Kutsyy (http://www.kutsyy.com)
!
!La matriz debe ser definida positiva y la subrutina identifica cuando no se cumple esta condición
!
!Parámetros de entrada:
!- Matriz por invertir: A (y sus dimensiones)
!
!Parámetros de salida: 
!- Matriz inversa: Ainv
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se debe tener 
!   cuidado con las variables y parámetros de entrada y salida de la misma>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine invChol(A,nA,Ainv)
implicit none
integer:: i,j,k
integer, intent(in):: nA
double precision, intent(in):: A(nA,nA)
double precision, intent(out):: Ainv(nA,nA)
double precision:: p(nA),ssum

Ainv=A
p=0.0

do i=1,nA
   do j=i,nA
      ssum=Ainv(i,j)
      do k=i-1,1,-1
         ssum=ssum-Ainv(i,k)*Ainv(j,k);
      end do
      if (i==j) then
         if (ssum <= 0.0) then
            write(*,*)' La matriz no es definida positiva!'
         end if
         p(i)=sqrt(ssum);
      else
         Ainv(j,i)=ssum/p(i)
      end if
   end do
end do

do i=1,nA
   Ainv(i,i)=1.0/p(i)
   do j=i+1,nA
      ssum=0.0
      do k=i,j-1
         ssum=ssum-Ainv(j,k)*Ainv(k,i)
      end do
      Ainv(j,i)=ssum/p(j)
   end do
end do

do i=1,nA
   do j=i+1,nA
      Ainv(i,j)=0.0
   end do
end do

do i=1,nA
   Ainv(i,i)=Ainv(i,i)*Ainv(i,i)
   do k=i+1,nA
      Ainv(i,i)=Ainv(i,i)+Ainv(k,i)*Ainv(k, i)
   end do
   do j=i+1,nA
      do k=j,nA
         Ainv(i,j)=Ainv(i,j)+Ainv(k,i)*Ainv(k,j)
      end do
   end do
end do

do i=1,nA
   do j=1,i-1
      Ainv(i,j)=Ainv(j,i)
   end do
end do

return
end subroutine invChol
!---------------------------------------------------------------------------------------------------

!---------------------------------------------------------------------------------------------------
!Subrutina deciles:
!Útil para calcular los deciles de las soluciones aceptadas.
!
!Parámetros de entrada:
!- Matriz de soluciones aceptadas: SolAcept
!- Número de soluciones aceptadas: acept
!
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se
!   debe tener cuidado con las variables y parámetros de entrada y salida de la misma.>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine deciles(SolAcept,acept,nVent,dec,ninterv)
implicit none
integer, intent(in):: ninterv
integer*8, intent(in):: acept,nVent
integer*8::i,j,tabla(ninterv,2)
double precision:: a,b,delta,k,Li,fai,fi,z(acept),SolAcept(acept,4),pcAcept
double precision, intent(out):: dec(ninterv)

!Tabla de frecuencias
z=SolAcept(:,3)
a=minval(z)
b=maxval(z)
delta=(b-a)/dble(ninterv)
tabla=0
dec=0.0
do i=1,acept
   do j=1,ninterv
      if (z(i) >= a+(j-1)*delta .and. z(i) < a+j*delta) then
         tabla(j,1)=tabla(j,1)+1
      end if
   end do
end do
tabla(ninterv,1)=tabla(ninterv,1)+1
do i=1,ninterv
   tabla(i,2)=sum(tabla(1:i,1))
end do

!Cálculo de deciles
do i=1,ninterv
   k=i*0.1*acept
   do j=1,ninterv
      if (k < tabla(j,2)) then
         exit
      end if
   end do
   if (j==1) then
      Li=a
      fai=0
      fi=tabla(j,1)
   else
      Li=a+(j-1)*delta
      fai=tabla(j-1,2)
      fi=tabla(j,1)
   end if   
   dec(i)=Li+(k-fai)*delta/fi
end do

!Porcentaje de soluciones aceptadas
pcAcept=dble(acept)/dble(nVent)

!Deciles en pantalla
write(*,10) nVent,acept,nVent-acept,pcAcept*100.0,(1.0-pcAcept)*100.0
if (acept < 1) return
write(*,11)
do i=1,ninterv
   write(*,12) i*ninterv, dec(i)
end do
write(*,13)

10 format('----------------------------------------------------------------------',/,&
         &'                 INFORMACION DE SOLUCIONES CALCULADAS                 ',/,&
         &'----------------------------------------------------------------------',/,&
         &'          Total de soluciones calculadas: ',i7,'                      ',/,&
         &'                    Soluciones aceptadas: ',i7,'                      ',/,&
         &'                   Soluciones rechazadas: ',i7,'                      ',/,&
         &'                Porcentaje de aceptacion: ',f7.2,' %                  ',/,&
         &'                   Porcentaje de rechazo: ',f7.2,' %                  ',/,&
         &'----------------------------------------------------------------------',/)

11 format('----------------------------------------------------------------------',/,&
         &'                   PERCENTILES (DECILES) CALCULADOS                   ',/,&
         &'----------------------------------------------------------------------')

12 format(3x,i3,'% de soluciones ubicadas a profundidades menores a ',f6.3,' km  ')

13 format('----------------------------------------------------------------------',/)

return
end subroutine deciles
!---------------------------------------------------------------------------------------------------
!Subrutina Esc_Result:
!Útil para almacenar resultados en archivos ASCII con formato XYZ.
!
!Parámetros de entrada:
!- Matriz de resultados: MatG
!- Número de datos en X y Y: Nx, Ny
!- Intervalos de muestro en direcciones X y Y: dx, dy
!- Coordenadas del punto inicial de la malla de observaciones: xini, yini
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se
!   debe tener cuidado con las variables y parámetros de entrada y salida de la misma.>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine Esc_Result(MatG,Nx,Ny,dx,dy,xini,yini)
implicit none
CHARACTER (LEN=1):: resp
CHARACTER (LEN=30):: ArchMAT,ArchXYZ
INTEGER:: i,j
INTEGER*8, INTENT(IN):: Nx,Ny
DOUBLE PRECISION, INTENT(IN):: dx,dy,xini,yini,MatG(Ny,Nx)
DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:):: X0,Y0,xXYZ,yXYZ,zXYZ

500 WRITE(*,13)
13 FORMAT(/,'Almacenar resultados en formato matricial? <S/N> ')
READ(*,*) resp

if (resp=='s'.OR.resp=='S') then
   WRITE(*,14)
14 FORMAT(/,'Archivo de salida: ')
   READ(*,*) ArchMAT
   OPEN(UNIT=200,FILE=ArchMAT,STATUS='replace',ACTION='write')
   do i=1,Ny
      WRITE(200,*)(MatG(i,j),j=1,Nx)
   end do
   CLOSE(200)
else if (resp=='n'.OR.resp=='N') then
   goto 600
else
   GOTO 500
end if

600 WRITE(*,15)
15 FORMAT(/,'Almacenar resultados en formato XYZ? <S/N> ')
READ(*,*) resp

if (resp=='s'.OR.resp=='S') then
   !Arreglos X, Y y Z
   ALLOCATE(X0(Nx),Y0(Ny))
   ALLOCATE(xXYZ(Nx*Ny),yXYZ(Nx*Ny),zXYZ(Nx*Ny))
   do i=1,Nx
      X0(i)=xini+(i-1)*dx
   end do
   do j=1,Ny
      Y0(j)=yini+(j-1)*dy
   end do
   do j=0,Nx-1
      do i=0,Ny-1
         xXYZ(i+1+Ny*j)=X0(j+1)
         yXYZ(i+1+Ny*j)=Y0(i+1)
         zXYZ(i+1+Ny*j)=MatG(i+1,j+1)
      end do
   end do
   DEALLOCATE(X0,Y0)
   WRITE(*,16)
16 FORMAT(/,'Archivo de salida: ')
   READ(*,*) ArchXYZ
   OPEN(UNIT=200,FILE=ArchXYZ,STATUS='replace',ACTION='write')
   do i=1,Nx*Ny
      WRITE(200,*) xXYZ(i),yXYZ(i),zXYZ(i)
   end do
   CLOSE(200)
   DEALLOCATE(xXYZ,yXYZ,zXYZ)
else if (resp=='n'.OR.resp=='N') then
   return
else
   GOTO 600
end if
return
end subroutine Esc_Result

!================================================================================================

!!!!!!_____------------------------------------Cálculo de eigenvalores y eigenvectores


! Esta es la subrutina para calcular el eigensistema por el metodo de jacobi
subroutine jacobi(A,d,v,nrot)
use nrtype
use nrutil
implicit none
integer*8, intent(out):: nrot
double PRECISION, dimension(:), intent(out) :: d
double PRECISION, dimension(:,:), intent(inout) :: A
double precision, dimension(:,:), intent(out) :: v
! Aqui se computan todos los eigenvalores y eigenvectores de una matriz simétrica real
! de n x n llamada a. En salida, los elementos de arriba de la diagonal de a son destruidos.
! d es un vector de longitud n que nos da los eigenvalores de a. v es una matriz n x n cuyas
! colimnas contienen, en salida, los eigenvectores normalizados de a. nrot nos regresa el numero de
! rotaciones de jacobi que se requirieron.
integer(I4B) :: i, ip, iq, n
real(sp) :: c,g,h,s,sm,t,tau,theta,tresh
real(sp), dimension(size(d)) :: b,z
n=assert_eq((/size(a,1),size(a,2),size(d),size(v,1),size(v,2)/),'jacobi')
call unit_matrix(v(:,:))   ! inicializa v a la matriz identidad
b(:)=get_diag(a(:,:))  !  inicializa b y d a la diagonal de a
d(:)=b(:)
z(:)=0.0   ! Éste vector acumulará términos
nrot=0
do i=1,50
sm=sum(abs(a),mask=upper_triangle(n,n))  !suma d elementos en la diagonal
if (sm== 0.0) return
! El regreso normal, que recae en convergencia cuadrática para calcular bajoflujo
tresh=merge(0.2_sp*sm/n**2,0.0_sp, i < 4 )
do ip=1,n-1
do iq=ip+1,n
g=100.0_sp*abs(a(ip,iq))
! Despues de cuatro barridas salta la rotacion si el elemento fuera de la diagonal es pequeño
if ((i >4) .and. (abs(d(ip))+g == abs(d(ip))) &
.and. (abs(d(iq))+g == abs(d(iq)))) then
a(ip,iq)=0.0
else if (abs(a(ip,iq)) > tresh) then
h=d(iq)-d(ip)
if (abs(h)+g == abs(h)) then
t=a(ip,iq)/h
else
theta=0.5_sp*h/a(ip,iq)
t=1.0_sp/(abs(theta)+sqrt(1.0_sp+theta**2))
if (theta < 0.0) t=-t
end if
c=1.0_sp/sqrt(1+t**2)
s=t*c
tau=s/(1.0_sp+c)
h=t*a(ip,iq)
z(ip)=z(ip)-h
z(iq)=z(iq)+h
d(ip)=d(ip)-h
d(iq)=d(iq)+h
a(ip,iq)=0.0
call jrotate(a(1:ip-1,ip),a(1:ip-1,iq))
call jrotate(a(ip,ip+1:iq-1),a(ip+1:iq-1,iq))
call jrotate(a(ip,iq+1:n),a(iq,iq+1:n))
call jrotate(v(:,ip),v(:,iq))
nrot=nrot+1
end if
end do
end do
b(:)=b(:)+z(:)
d(:)=b(:)
z(:)=0.0
end do
call nrerror('Demasiadas iteraciones del jacobi')
contains

subroutine jrotate(a1,a2)
double precision, dimension(:), intent(inout) :: a1,a2
double precision, dimension(size(a1)) :: wk1
wk1(:)=a1(:)
a1(:)=a1(:)-s*(a2(:)+a1(:)*tau)
a2(:)=a2(:)+s*(wk1(:)-a2(:)*tau)
end subroutine jrotate
end subroutine jacobi

!-------------Subrutina acomoda eigen vectores y eigen valores--------------------------------

subroutine eigsrt(d,v)
use nrtype
use nrutil
implicit none
double precision, dimension(:),intent(inout) :: d
double precision, dimension(:,:), intent(inout) :: v

!  Dados los eigenvalores d y los eigenvectores v como vienen de salida de jacobi
!  los acomoda de manera descendente y reacomoda las columnas de v correspondientemente

integer*8 :: i,j,n
n=assert_eq(size(d),size(v,1),size(v,2),'eigsrt')
do i=1,n-1
j=imaxloc_r(d(i:n))+i-1
if (j /= i) then
call swap_r(d(i),d(j))
call swap_rv(v(:,i),v(:,j))
end if
end do
end subroutine eigsrt





!---------------------------------------------------------------------------------------------------

!---------------------------------------------------------------------------------------------------
!Subrutina EscSolgnuplot:
!Útil para graficar en gnuplot, las soluciones numéricas de la Ec. Onda 2D. en un instante
!determinado por el paso de tiempo e intervalo de muestreo temporal. Sólo se grafica el dominio
!principal.
!
!Parámetros de entrada:
!- Solución numérica (arreglo 2D): A
!- Dimensiones de Sx: Nx,Nz
!  * Nota: Se pasa el arreglo con los puntos de malla agregados para la PML
!- Número de puntos de la PML: NPML
!- Paso de tiempo en que se graficará la solución: k
!- Intervalo de muestreo temporal: dt
!
!<<<Esta subrutina ha sido adaptada para su uso dentro del módulo Euler, por lo que se debe tener 
!   cuidado con las variables y parámetros de entrada y salida de la misma>>>
!
!Elaboró: Mauricio Nava Flores
!---------------------------------------------------------------------------------------------------
subroutine EscSolgnuplot(A,Nx,Nz,NPML,k,h,dt,pre)
implicit none
 character(len=100):: linea1,linea2
 character(len=50):: solucionSx
 character(len=5):: extension
 character(len=15):: N_Arch,xmin,xMax,zmin,zMax,marcasx(3),marcasz(3),dmarcx,dmarcz
 character(len=2), intent(in):: pre
integer, intent(in):: Nx,Nz,k,NPML
integer:: reg=1
double precision, intent(in):: dt,h
double precision, dimension(Nz+NPML-1,Nx+2*(NPML-1)), intent(in):: A
double precision, dimension(Nz,Nx):: B
double precision:: Maximo

B=A(1:Nz,NPML:NPML+Nx-1)

!Normalización de la solución
if (pre=='Sx') then
   Maximo=2568818.734
else if(pre=='Vx') then
   Maximo=0.233
else if(pre=='Vz') then
   Maximo=0.533
end if

B=B/(Maximo*0.25)

extension='.png'
if (k*dt>=0.0 .and. k*dt<10.0) then
   write(N_Arch,'(f5.3)') k*dt
elseif (k*dt>=10.0 .and. k*dt<100.0) then
   write(N_Arch,'(f6.3)') k*dt
elseif (k*dt>=100.0 .and. k*dt<1000.0) then
   write(N_Arch,'(f7.3)') k*dt
elseif (k*dt>=1000.0 .and. k*dt<10000.0) then
   write(N_Arch,'(f8.3)') k*dt
end if
solucionSx=trim(pre)//trim('_')//trim(N_Arch)//trim('s')//trim(extension)

!Solución en archivo binario
open(100,file='Solucion.bin',status='replace',access='direct',action='write',recl=8*Nx*Nz)
write(100,rec=reg) B
 close(100)

write(xmin,'(f3.1)') 0.0
write(zmin,'(f3.1)') 0.0

if ((Nx-1)*h>=100.0 .and. (Nx-1)*h<1000.0) then
   write(xMax,'(f6.2)') (Nx-1)*h
elseif ((Nx-1)*h>=1000.0 .and. (Nx-1)*h<10000.0) then
   write(xMax,'(f7.2)') (Nx-1)*h
elseif ((Nx-1)*h>=10000.0 .and. (Nx-1)*h<100000.0) then
   write(xMax,'(f8.2)') (Nx-1)*h
end if

if ((Nz-1)*h>=100.0 .and. (Nz-1)*h<1000.0) then
   write(zMax,'(f6.2)') (Nz-1)*h
elseif ((Nz-1)*h>=1000.0 .and. (Nz-1)*h<10000.0) then
   write(zMax,'(f7.2)') (Nz-1)*h
elseif ((Nz-1)*h>=10000.0 .and. (Nz-1)*h<100000.0) then
   write(zMax,'(f8.2)') (Nz-1)*h
end if

dmarcx='2500.0'
dmarcz='2500.0'
marcasx(1)=trim(xmin)
marcasx(2)=trim(dmarcx)
marcasx(3)=trim(xMax)
marcasz(1)=trim(zmin)
marcasz(2)=trim(dmarcz)
marcasz(3)=trim(zMax)

if (pre=='Sx') then
   linea1='set title "{/Symbol s}_x" font "Arial black, 24"'
   linea2='set terminal png size 1260,720 enhanced crop font "Arial black 12"'
else if (pre=='Vx') then
   linea1='set title "V_x" font "Arial black, 24"'
   linea2='set terminal png size 1260,500 enhanced crop font "Arial black 12"'
else if (pre=='Vz') then
   linea1='set title "V_z" font "Arial black, 24"'
   linea2='set terminal png size 1260,500 enhanced crop font "Arial black 12"'
end if

open(200,file='script.gnu',status='replace',action='write')
write(200,10)trim(xmin),trim(xMax),trim(zMax),trim(zmin),trim(linea2),trim(marcasx(1)),   &
             trim(marcasx(2)),trim(marcasx(3)),trim(marcasz(1)),trim(marcasz(2)),trim(marcasz(3)), &
             trim(linea1),trim(SolucionSx),Nx,Nz,h,h
 close(200)

call system('gnuplot script.gnu')
call system('rm Solucion.bin script.gnu')

10 format('#Script gnuplot para graficar las soluciones de la propagación',/,&
&'set lmargin 0.1',/,&
&'set rmargin 0.1',/,&
&'set view map',/,&
&'set size ratio -1',/,&
&"set palette model RGB defined (-1 'blue', 0 'white', 1 'red')",/,&
&'set cbrange [-1.0:1.0]',/,&
&'set cbtics scale 0.75 font "Arial, 10"',/,&
&'set cblabel "Amplitudes normalizadas" font "Arial black 16" rotate by -90',/,&
&'set xrange [',a,':',a,']',/,&
&'set yrange [',a,':',a,']',/,&
& a,/,&
&'set xtics out scale 0.75 nomirror ',a,',',a,',',a,' font "Arial, 10"',/,&
&'set ytics out scale 0.75 nomirror ',a,',',a,',',a,' font "Arial, 10"',/,&
& a,/,&
&'set output "',a,'"',/,&
&'set xlabel "Distancia [m]" font "Arial black 16"',/,&
&'set ylabel "Profundidad [m]" font "Arial black 16"',/,&
&'splot "Solucion.bin" binary array=(',i6,',',i6,') scan=yx format="%float64"  \', /,&
&'      dx=',f7.3,' dy=',f7.3,' origin=(0,0,0) w pm3d t ""',/,&
&'reset',/)
return
end subroutine EscSolgnuplot
!---------------------------------------------------------------------------------------------------

end module Eulermio
!---------------------------------------------------------------------------------------------------
